@@ -26,9 +26,7 @@
 #include "qwaylandhardwareintegration_p.h"
 #include "qwaylandtextinputv1_p.h"
 #include "qwaylandtextinputv2_p.h"
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-#include "qwaylandtextinputv4_p.h"
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+#include "qwaylandtextinputv3_p.h"
 #include "qwaylandinputcontext_p.h"
 #include "qwaylandinputmethodcontext_p.h"
 
@@ -47,13 +45,13 @@
 
 #include <QtWaylandClient/private/qwayland-text-input-unstable-v1.h>
 #include <QtWaylandClient/private/qwayland-text-input-unstable-v2.h>
-#include <QtWaylandClient/private/qwayland-text-input-unstable-v4-wip.h>
+#include <QtWaylandClient/private/qwayland-text-input-unstable-v3.h>
 #include <QtWaylandClient/private/qwayland-wp-primary-selection-unstable-v1.h>
 #include <QtWaylandClient/private/qwayland-qt-text-input-method-unstable-v1.h>
 #include <QtWaylandClient/private/qwayland-fractional-scale-v1.h>
 #include <QtWaylandClient/private/qwayland-viewporter.h>
 #include <QtWaylandClient/private/qwayland-cursor-shape-v1.h>
-#include <QtWaylandClient/private/qwayland-qt-toplevel-drag-v1.h>
+#include <QtWaylandClient/private/qwayland-xdg-toplevel-drag-v1.h>
 
 #include <QtCore/private/qcore_unix_p.h>
 
@@ -279,7 +277,7 @@ struct ::wl_region *QWaylandDisplay::createRegion(const QRegion &qregion)
 
 ::wl_subsurface *QWaylandDisplay::createSubSurface(QWaylandWindow *window, QWaylandWindow *parent)
 {
-    if (!mSubCompositor) {
+    if (!mGlobals.subCompositor) {
         qCWarning(lcQpaWayland) << "Can't create subsurface, not supported by the compositor.";
         return nullptr;
     }
@@ -288,18 +286,18 @@ struct ::wl_region *QWaylandDisplay::createRegion(const QRegion &qregion)
     Q_ASSERT(parent->wlSurface());
     Q_ASSERT(window->wlSurface());
 
-    return mSubCompositor->get_subsurface(window->wlSurface(), parent->wlSurface());
+    return mGlobals.subCompositor->get_subsurface(window->wlSurface(), parent->wlSurface());
 }
 
 ::wp_viewport *QWaylandDisplay::createViewport(QWaylandWindow *window)
 {
-    if (!mViewporter) {
+    if (!mGlobals.viewporter) {
         qCWarning(lcQpaWayland) << "Can't create wp_viewport, not supported by the compositor.";
         return nullptr;
     }
 
     Q_ASSERT(window->wlSurface());
-    return mViewporter->get_viewport(window->wlSurface());
+    return mGlobals.viewporter->get_viewport(window->wlSurface());
 }
 
 QWaylandShellIntegration *QWaylandDisplay::shellIntegration() const
@@ -366,9 +364,6 @@ QWaylandDisplay::~QWaylandDisplay(void)
     }
     qDeleteAll(mWaitingScreens);
 
-#if QT_CONFIG(wayland_datadevice)
-    mDndSelectionHandler.reset();
-#endif
 #if QT_CONFIG(cursor)
     mCursorThemes.clear();
 #endif
@@ -376,6 +371,8 @@ QWaylandDisplay::~QWaylandDisplay(void)
     if (m_frameEventQueue)
         wl_event_queue_destroy(m_frameEventQueue);
 
+    mGlobals = {};
+
     if (mDisplay)
         wl_display_disconnect(mDisplay);
 }
@@ -389,6 +386,8 @@ bool QWaylandDisplay::initialize()
 
     forceRoundTrip();
 
+    emit connected();
+
     if (!mWaitingScreens.isEmpty()) {
         // Give wl_output.done and zxdg_output_v1.done events a chance to arrive
         forceRoundTrip();
@@ -432,37 +431,18 @@ void QWaylandDisplay::reconnect()
     mShm.reset();
     mCursorThemes.clear();
     mCursor.reset();
-    mDndSelectionHandler.reset();
-    mWindowExtension.reset();
-    mSubCompositor.reset();
-    mTouchExtension.reset();
-    mQtKeyExtension.reset();
-    mWindowManagerIntegration.reset();
-    mTabletManager.reset();
-    mPointerGestures.reset();
-#if QT_CONFIG(wayland_client_primary_selection)
-    mPrimarySelectionManager.reset();
-#endif
-    mTextInputMethodManager.reset();
-    mTextInputManagerv1.reset();
-    mTextInputManagerv2.reset();
-    mTextInputManagerv4.reset();
-    mHardwareIntegration.reset();
-    mXdgOutputManager.reset();
-    mViewporter.reset();
-    mFractionalScaleManager.reset();
-    mCursorShapeManager.reset();
-    mXdgToplevelDragManager.reset();
+
+    mGlobals = GlobalHolder();
 
     mWaylandIntegration->reset();
 
     qDeleteAll(std::exchange(mInputDevices, {}));
     mLastInputDevice = nullptr;
 
-    for (const RegistryGlobal &global : mGlobals) {
+    for (const RegistryGlobal &global : mRegistryGlobals) {
         emit globalRemoved(global);
     }
-    mGlobals.clear();
+    mRegistryGlobals.clear();
 
     mLastInputSerial = 0;
     mLastInputWindow.clear();
@@ -500,8 +480,6 @@ void QWaylandDisplay::reconnect()
         wl_event_queue_destroy(m_frameEventQueue);
     initEventThread();
 
-    emit reconnected();
-
     auto needsRecreate = [](QPlatformWindow *window) {
         return window && !static_cast<QWaylandWindow *>(window)->wlSurface();
     };
@@ -575,16 +553,18 @@ void QWaylandDisplay::blockingReadEvents()
 void QWaylandDisplay::checkTextInputProtocol()
 {
     QStringList tips, timps; // for text input protocols and text input manager protocols
+    // zwp_text_input_v2 is preferred over zwp_text_input_v3 because:
+    // - Currently, v3 is not as feature rich as v2.
+    // - While v2 is not upstreamed, it is well supported by KWin since Plasma 5 and Plasma
+    //   Mobile uses some v2 only.
     tips << QLatin1String(QtWayland::qt_text_input_method_v1::interface()->name)
          << QLatin1String(QtWayland::zwp_text_input_v2::interface()->name)
+         << QLatin1String(QtWayland::zwp_text_input_v3::interface()->name)
          << QLatin1String(QtWayland::zwp_text_input_v1::interface()->name);
     timps << QLatin1String(QtWayland::qt_text_input_method_manager_v1::interface()->name)
           << QLatin1String(QtWayland::zwp_text_input_manager_v2::interface()->name)
+          << QLatin1String(QtWayland::zwp_text_input_manager_v3::interface()->name)
           << QLatin1String(QtWayland::zwp_text_input_manager_v1::interface()->name);
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-    tips << QLatin1String(QtWayland::zwp_text_input_v4::interface()->name);
-    timps << QLatin1String(QtWayland::zwp_text_input_manager_v4::interface()->name);
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
 
     QString tiProtocols = QString::fromLocal8Bit(qgetenv("QT_WAYLAND_TEXT_INPUT_PROTOCOL"));
     qCDebug(lcQpaWayland) << "QT_WAYLAND_TEXT_INPUT_PROTOCOL=" << tiProtocols;
@@ -638,7 +618,7 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
     if (interface == QLatin1String(QtWayland::wl_output::interface()->name)) {
         mWaitingScreens << mWaylandIntegration->createPlatformScreen(this, version, id);
     } else if (interface == QLatin1String(QtWayland::wl_compositor::interface()->name)) {
-        mCompositor.init(registry, id, qMin((int)version, 4));
+        mCompositor.init(registry, id, qMin((int)version, 6));
     } else if (interface == QLatin1String(QWaylandShm::interface()->name)) {
         mShm.reset(new QWaylandShm(this, version, id));
     } else if (interface == QLatin1String(QWaylandInputDevice::interface()->name)) {
@@ -646,62 +626,69 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
         mInputDevices.append(inputDevice);
 #if QT_CONFIG(wayland_datadevice)
     } else if (interface == QLatin1String(QWaylandDataDeviceManager::interface()->name)) {
-        mDndSelectionHandler.reset(new QWaylandDataDeviceManager(this, version, id));
+        mGlobals.dndSelectionHandler.reset(new QWaylandDataDeviceManager(this, version, id));
 #endif
     } else if (interface == QLatin1String(QtWayland::qt_surface_extension::interface()->name)) {
-        mWindowExtension.reset(new QtWayland::qt_surface_extension(registry, id, 1));
+        mGlobals.surfaceExtension.reset(new QtWayland::qt_surface_extension(registry, id, 1));
     } else if (interface == QLatin1String(QtWayland::wl_subcompositor::interface()->name)) {
-        mSubCompositor.reset(new QtWayland::wl_subcompositor(registry, id, 1));
+        mGlobals.subCompositor.reset(new QtWayland::wl_subcompositor(registry, id, 1));
     } else if (interface == QLatin1String(QWaylandTouchExtension::interface()->name)) {
-        mTouchExtension.reset(new QWaylandTouchExtension(this, id));
+        mGlobals.touchExtension.reset(new QWaylandTouchExtension(this, id));
     } else if (interface == QLatin1String(QWaylandQtKeyExtension::interface()->name)) {
-        mQtKeyExtension.reset(new QWaylandQtKeyExtension(this, id));
+        mGlobals.qtKeyExtension.reset(new QWaylandQtKeyExtension(this, id));
 #if QT_CONFIG(tabletevent)
     } else if (interface == QLatin1String(QWaylandTabletManagerV2::interface()->name)) {
-        mTabletManager.reset(new QWaylandTabletManagerV2(this, id, qMin(1, int(version))));
+        mGlobals.tabletManager.reset(new QWaylandTabletManagerV2(this, id, qMin(1, int(version))));
+        for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
+            inputDevice->setTabletSeat(
+                    new QWaylandTabletSeatV2(mGlobals.tabletManager.get(), inputDevice));
 #endif
     } else if (interface == QLatin1String(QWaylandPointerGestures::interface()->name)) {
-        mPointerGestures.reset(new QWaylandPointerGestures(this, id, 1));
+        mGlobals.pointerGestures.reset(new QWaylandPointerGestures(this, id, 1));
 #if QT_CONFIG(wayland_client_primary_selection)
     } else if (interface == QLatin1String(QWaylandPrimarySelectionDeviceManagerV1::interface()->name)) {
-        mPrimarySelectionManager.reset(new QWaylandPrimarySelectionDeviceManagerV1(this, id, 1));
+        mGlobals.primarySelectionManager.reset(
+                new QWaylandPrimarySelectionDeviceManagerV1(this, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
-            inputDevice->setPrimarySelectionDevice(mPrimarySelectionManager->createDevice(inputDevice));
+            inputDevice->setPrimarySelectionDevice(
+                    mGlobals.primarySelectionManager->createDevice(inputDevice));
 #endif
     } else if (interface == QLatin1String(QtWayland::qt_text_input_method_manager_v1::interface()->name)
             && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
         qCDebug(lcQpaWayland) << "text input: register qt_text_input_method_manager_v1";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputManagerv1.reset();
-            mTextInputManagerv2.reset();
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            mTextInputManagerv4.reset();
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+            mGlobals.textInputManagerv1.reset();
+            mGlobals.textInputManagerv2.reset();
+            mGlobals.textInputManagerv3.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInput(nullptr);
         }
 
-        mTextInputMethodManager.reset(new QtWayland::qt_text_input_method_manager_v1(registry, id, 1));
+        mGlobals.textInputMethodManager.reset(
+                new QtWayland::qt_text_input_method_manager_v1(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
-            inputDevice->setTextInputMethod(new QWaylandTextInputMethod(this, mTextInputMethodManager->get_text_input_method(inputDevice->wl_seat())));
+            inputDevice->setTextInputMethod(new QWaylandTextInputMethod(
+                    this,
+                    mGlobals.textInputMethodManager->get_text_input_method(
+                            inputDevice->wl_seat())));
         mWaylandIntegration->reconfigureInputContext();
         mTextInputManagerIndex = mTextInputManagerList.indexOf(interface);
     } else if (interface == QLatin1String(QtWayland::zwp_text_input_manager_v1::interface()->name)
                && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
         qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v1";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputMethodManager.reset();
-            mTextInputManagerv2.reset();
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            mTextInputManagerv4.reset();
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+            mGlobals.textInputMethodManager.reset();
+            mGlobals.textInputManagerv2.reset();
+            mGlobals.textInputManagerv3.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInputMethod(nullptr);
         }
 
-        mTextInputManagerv1.reset(new QtWayland::zwp_text_input_manager_v1(registry, id, 1));
+        mGlobals.textInputManagerv1.reset(
+                new QtWayland::zwp_text_input_manager_v1(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices)) {
-            auto textInput = new QWaylandTextInputv1(this, mTextInputManagerv1->create_text_input());
+            auto textInput =
+                    new QWaylandTextInputv1(this, mGlobals.textInputManagerv1->create_text_input());
             textInput->setSeat(inputDevice->wl_seat());
             inputDevice->setTextInput(textInput);
         }
@@ -712,62 +699,66 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
             && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
         qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v2";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputMethodManager.reset();
-            mTextInputManagerv1.reset();
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            mTextInputManagerv4.reset();
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+            mGlobals.textInputMethodManager.reset();
+            mGlobals.textInputManagerv1.reset();
+            mGlobals.textInputManagerv3.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInputMethod(nullptr);
         }
 
-        mTextInputManagerv2.reset(new QtWayland::zwp_text_input_manager_v2(registry, id, 1));
+        mGlobals.textInputManagerv2.reset(
+                new QtWayland::zwp_text_input_manager_v2(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
-            inputDevice->setTextInput(new QWaylandTextInputv2(this, mTextInputManagerv2->get_text_input(inputDevice->wl_seat())));
+            inputDevice->setTextInput(new QWaylandTextInputv2(
+                    this, mGlobals.textInputManagerv2->get_text_input(inputDevice->wl_seat())));
         mWaylandIntegration->reconfigureInputContext();
         mTextInputManagerIndex = mTextInputManagerList.indexOf(interface);
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-    } else if (interface == QLatin1String(QtWayland::zwp_text_input_manager_v4::interface()->name)
+    } else if (interface == QLatin1String(QtWayland::zwp_text_input_manager_v3::interface()->name)
             && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
-        qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v4";
+        qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v3";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputMethodManager.reset();
-            mTextInputManagerv2.reset();
+            mGlobals.textInputMethodManager.reset();
+            mGlobals.textInputManagerv2.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInputMethod(nullptr);
         }
 
-        mTextInputManagerv4.reset(new QtWayland::zwp_text_input_manager_v4(registry, id, 1));
+        mGlobals.textInputManagerv3.reset(
+                new QtWayland::zwp_text_input_manager_v3(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
-            inputDevice->setTextInput(new QWaylandTextInputv4(this, mTextInputManagerv4->get_text_input(inputDevice->wl_seat())));
+            inputDevice->setTextInput(new QWaylandTextInputv3(
+                    this, mGlobals.textInputManagerv3->get_text_input(inputDevice->wl_seat())));
+
         mWaylandIntegration->reconfigureInputContext();
         mTextInputManagerIndex = mTextInputManagerList.indexOf(interface);
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
-    } else if (interface == QLatin1String(QWaylandHardwareIntegration::interface()->name)) {
+    }else if (interface == QLatin1String(QWaylandHardwareIntegration::interface()->name)) {
         bool disableHardwareIntegration = qEnvironmentVariableIntValue("QT_WAYLAND_DISABLE_HW_INTEGRATION");
         if (!disableHardwareIntegration) {
-            mHardwareIntegration.reset(new QWaylandHardwareIntegration(registry, id));
+            mGlobals.hardwareIntegration.reset(new QWaylandHardwareIntegration(registry, id));
             // make a roundtrip here since we need to receive the events sent by
             // qt_hardware_integration before creating windows
             forceRoundTrip();
         }
     } else if (interface == QLatin1String(QWaylandXdgOutputManagerV1::interface()->name)) {
-        mXdgOutputManager.reset(new QWaylandXdgOutputManagerV1(this, id, version));
+        mGlobals.xdgOutputManager.reset(new QWaylandXdgOutputManagerV1(this, id, version));
         for (auto *screen : std::as_const(mWaitingScreens))
             screen->initXdgOutput(xdgOutputManager());
     } else if (interface == QLatin1String(QtWayland::wp_fractional_scale_manager_v1::interface()->name)) {
-        mFractionalScaleManager.reset(new QtWayland::wp_fractional_scale_manager_v1(registry, id, 1));
+        mGlobals.fractionalScaleManager.reset(
+                new QtWayland::wp_fractional_scale_manager_v1(registry, id, 1));
     } else if (interface == QLatin1String("wp_viewporter")) {
-        mViewporter.reset(new QtWayland::wp_viewporter(registry, id, qMin(1u, version)));
+        mGlobals.viewporter.reset(new QtWayland::wp_viewporter(registry, id, qMin(1u, version)));
     } else if (interface == QLatin1String(QtWayland::wp_cursor_shape_manager_v1::interface()->name)) {
-        mCursorShapeManager.reset(new QtWayland::wp_cursor_shape_manager_v1(registry, id, std::min(1u, version)));
+        mGlobals.cursorShapeManager.reset(
+                new QtWayland::wp_cursor_shape_manager_v1(registry, id, std::min(1u, version)));
     } else if (
-            interface == QLatin1String(QtWayland::qt_toplevel_drag_manager_v1::interface()->name)) {
-        mXdgToplevelDragManager.reset(new QtWayland::qt_toplevel_drag_manager_v1(registry, id, 1));
+            interface == QLatin1String(QtWayland::xdg_toplevel_drag_manager_v1::interface()->name)) {
+        mGlobals.xdgToplevelDragManager.reset(
+                new QtWayland::xdg_toplevel_drag_manager_v1(registry, id, 1));
     }
 
-    mGlobals.append(RegistryGlobal(id, interface, version, registry));
-    emit globalAdded(mGlobals.back());
+    mRegistryGlobals.append(RegistryGlobal(id, interface, version, registry));
+    emit globalAdded(mRegistryGlobals.back());
 
     const auto copy = mRegistryListeners; // be prepared for listeners unregistering on notification
     for (Listener l : copy)
@@ -776,8 +767,8 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
 
 void QWaylandDisplay::registry_global_remove(uint32_t id)
 {
-    for (int i = 0, ie = mGlobals.size(); i != ie; ++i) {
-        RegistryGlobal &global = mGlobals[i];
+    for (int i = 0, ie = mRegistryGlobals.size(); i != ie; ++i) {
+        RegistryGlobal &global = mRegistryGlobals[i];
         if (global.id == id) {
             if (global.interface == QLatin1String(QtWayland::wl_output::interface()->name)) {
                 for (auto *screen : mWaitingScreens) {
@@ -799,39 +790,37 @@ void QWaylandDisplay::registry_global_remove(uint32_t id)
                 }
             }
             if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v1::interface()->name)) {
-                mTextInputManagerv1.reset();
+                mGlobals.textInputManagerv1.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInput(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
             if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v2::interface()->name)) {
-                mTextInputManagerv2.reset();
+                mGlobals.textInputManagerv2.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInput(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v4::interface()->name)) {
-                mTextInputManagerv4.reset();
+            if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v3::interface()->name)) {
+                mGlobals.textInputManagerv3.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInput(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
             if (global.interface == QLatin1String(QtWayland::qt_text_input_method_manager_v1::interface()->name)) {
-                mTextInputMethodManager.reset();
+                mGlobals.textInputMethodManager.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInputMethod(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
 #if QT_CONFIG(wayland_client_primary_selection)
             if (global.interface == QLatin1String(QtWayland::zwp_primary_selection_device_manager_v1::interface()->name)) {
-                mPrimarySelectionManager.reset();
+                mGlobals.primarySelectionManager.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setPrimarySelectionDevice(nullptr);
             }
 #endif
-            emit globalRemoved(mGlobals.takeAt(i));
+            emit globalRemoved(mRegistryGlobals.takeAt(i));
             break;
         }
     }
@@ -839,7 +828,7 @@ void QWaylandDisplay::registry_global_remove(uint32_t id)
 
 bool QWaylandDisplay::hasRegistryGlobal(QStringView interfaceName) const
 {
-    for (const RegistryGlobal &global : mGlobals)
+    for (const RegistryGlobal &global : mRegistryGlobals)
         if (global.interface == interfaceName)
             return true;
 
@@ -850,8 +839,9 @@ void QWaylandDisplay::addRegistryListener(RegistryListener listener, void *data)
 {
     Listener l = { listener, data };
     mRegistryListeners.append(l);
-    for (int i = 0, ie = mGlobals.size(); i != ie; ++i)
-        (*l.listener)(l.data, mGlobals[i].registry, mGlobals[i].id, mGlobals[i].interface, mGlobals[i].version);
+    for (int i = 0, ie = mRegistryGlobals.size(); i != ie; ++i)
+        (*l.listener)(l.data, mRegistryGlobals[i].registry, mRegistryGlobals[i].id,
+                      mRegistryGlobals[i].interface, mRegistryGlobals[i].version);
 }
 
 void QWaylandDisplay::removeListener(RegistryListener listener, void *data)
@@ -862,16 +852,6 @@ void QWaylandDisplay::removeListener(RegistryListener listener, void *data)
     mRegistryListeners.erase(iter, mRegistryListeners.end());
 }
 
-uint32_t QWaylandDisplay::currentTimeMillisec()
-{
-    //### we throw away the time information
-    struct timeval tv;
-    int ret = gettimeofday(&tv, nullptr);
-    if (ret == 0)
-        return tv.tv_sec*1000 + tv.tv_usec/1000;
-    return 0;
-}
-
 void QWaylandDisplay::forceRoundTrip()
 {
      wl_display_roundtrip(mDisplay);
@@ -962,7 +942,7 @@ void QWaylandDisplay::handleWaylandSync()
     // handleWindowActivated() calls immediately.
     QWindow *activeWindow = mActiveWindows.empty() ? nullptr : mActiveWindows.last()->window();
     if (activeWindow != QGuiApplication::focusWindow())
-        QWindowSystemInterface::handleWindowActivated(activeWindow);
+        QWindowSystemInterface::handleFocusWindowChanged(activeWindow);
 
     if (!activeWindow) {
         if (lastInputDevice()) {
